@@ -1,6 +1,6 @@
 /* Debug printing functions.
 
-   Copyright (C) 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2014-2020 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -19,10 +19,23 @@
 
 #include "defs.h"
 
-/* See common/common-debug.h.  */
+#include "debug.h"
+
+/* See gdbsupport/common-debug.h.  */
 
 void
 debug_vprintf (const char *fmt, va_list ap)
 {
   vfprintf_unfiltered (gdb_stdlog, fmt, ap);
 }
+
+/* See debug.h.  */
+
+void
+debug_prefixed_vprintf (const char *module, const char *func, const char *format,
+			va_list args)
+{
+  debug_printf ("[%s] %s: ", module, func);
+  debug_vprintf (format, args);
+  debug_printf ("\n");
+}
